# -*- coding: utf-8 -*-
from zope import schema
from zope.component import adapts
from zope.interface import alsoProvides
from zope.interface import implements
from collective.behavior.talcondition import _
from collective.behavior.talcondition.utils import evaluateExpressionFor
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from plone.autoform import directives as form
from z3c.form.browser.checkbox import CheckBoxFieldWidget


class ITALCondition(model.Schema):

    form.widget('tal_condition', size=80)
    tal_condition = schema.TextLine(
        title=_(u'TAL condition expression'),
        description=_(u'Enter a TAL expression that once evaluated '
                      'will return \'True\' if content should be '
                      'available. Elements \'member\', \'context\' '
                      'and \'portal\' are available for the '
                      'expression.'),
        required=False,
        default=u'',
    )

    form.widget('roles_bypassing_talcondition', CheckBoxFieldWidget, multiple='multiple', size=15)
    roles_bypassing_talcondition = schema.Set(
        title=_(u'Roles that will bypass the TAL condition'),
        description=_(u'Choose the different roles for which the TAL '
                      'condition will not be evaluated and always '
                      'considered \'True\'.'),
        required=False,
        value_type=schema.Choice(vocabulary='plone.app.vocabularies.Roles'),
    )

    def evaluate(self):
        """Evaluate the condition and returns True or False."""

alsoProvides(ITALCondition, IFormFieldProvider)


class TALCondition(object):
    """
    """

    implements(ITALCondition)
    adapts(IDexterityContent)

    def __init__(self, context):
        self.context = context

    def get_tal_condition(self):
        return getattr(self.context, 'tal_condition', '')

    def set_tal_condition(self, value):
        self.context.tal_condition = value

    tal_condition = property(get_tal_condition, set_tal_condition)

    def get_roles_bypassing_talcondition(self):
        return getattr(self.context, 'roles_bypassing_talcondition', [])

    def set_roles_bypassing_talcondition(self, value):
        self.context.roles_bypassing_talcondition = value

    roles_bypassing_talcondition = property(get_roles_bypassing_talcondition, set_roles_bypassing_talcondition)

    def evaluate(self, extra_expr_ctx={}):
        return evaluateExpressionFor(self, extra_expr_ctx=extra_expr_ctx)



# -*- coding: utf-8 -*-


def isNotCurrentProfile(context):
    return context.readDataFile("collectivebehaviortalcondition_marker.txt") is None


def post_install(context):
    """Post install script"""
    if isNotCurrentProfile(context):
        return



# -*- coding: utf-8 -*-
from plone.app.testing import login
from plone.app.testing import TEST_USER_NAME
from collective.behavior.talcondition import PLONE_VERSION
from collective.behavior.talcondition.behavior import ITALCondition
from collective.behavior.talcondition.testing import IntegrationTestCase
from collective.behavior.talcondition.interfaces import ITALConditionable


class TestBehavior(IntegrationTestCase):

    def setUp(self):
        """ """
        super(TestBehavior, self).setUp()
        # create a testitem
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id='testitem',
                                  type_name='testtype',
                                  title='Test type')
        self.testitem = self.portal.testitem
        self.adapted = ITALCondition(self.testitem)

    def test_behavior(self):
        """Test that once enabled, the behavior do the job."""
        if PLONE_VERSION < 5:
            # in Plone 4, a behavior attribute is not set (until saved ?)
            self.assertFalse(hasattr(self.testitem, 'tal_condition'))
        # it has a 'tal_condition' attribute
        self.assertTrue(hasattr(self.adapted, 'tal_condition'))
        self.assertTrue(ITALConditionable.providedBy(self.portal.testitem))
        # set a tal_condition and evaluate
        # this is True
        self.adapted.tal_condition = u"python:context.portal_type=='testtype'"
        self.assertTrue(self.adapted.evaluate())
        # this is False
        self.adapted.tal_condition = u"python:context.portal_type=='unexisting_portal_type'"
        self.assertFalse(self.adapted.evaluate())

    def test_wrong_condition(self):
        """In case the condition is wrong, it just returns False
           and a message is added to the Zope log."""
        # using a wrong expression does not break anything
        self.adapted.tal_condition = u'python: context.some_unexisting_method()'
        self.assertFalse(self.adapted.evaluate())

    def test_evaluate_extra_expr_ctx(self):
        """The 'evaluate' method can receive a 'extra_expr_ctx' dict
           that will extend the TAL expression context."""
        self.adapted.tal_condition = "python: value == '122'"
        self.assertFalse(self.adapted.evaluate())
        self.assertTrue(self.adapted.evaluate(extra_expr_ctx={'value': '122'}))



# -*- coding: utf-8 -*-
"""Base module for unittesting."""

from plone import api
from plone.app.robotframework.testing import AUTOLOGIN_LIBRARY_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import login
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_NAME
from plone.testing import z2

import unittest

import collective.behavior.talcondition


class CollectiveBehaviorTalconditionLayer(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)
    products = ('collective.behavior.talcondition', )

    def setUpZope(self, app, configurationContext):
        """Set up Zope."""
        # Load ZCML
        self.loadZCML(package=collective.behavior.talcondition,
                      name='testing.zcml')
        for p in self.products:
            z2.installProduct(app, p)

    def setUpPloneSite(self, portal):
        """Set up Plone."""
        # Set default chain for plone.app.contenttypes
        wftool = portal['portal_workflow']
        wftool.setDefaultChain('simple_publication_workflow')

        # Install into Plone site using portal_setup
        applyProfile(portal, 'collective.behavior.talcondition:testing')

        try:
            applyProfile(portal, 'plone.app.contenttypes:plone-content')
        except KeyError:
            # BBB Plone 4
            pass

        # Login and create some test content
        setRoles(portal, TEST_USER_ID, ['Manager'])
        login(portal, TEST_USER_NAME)
        api.content.create(container=portal, type='Folder', id='folder')

        # Commit so that the test browser sees these objects
        import transaction
        transaction.commit()

    def tearDownZope(self, app):
        """Tear down Zope."""
        for p in reversed(self.products):
            z2.uninstallProduct(app, p)

FIXTURE = CollectiveBehaviorTalconditionLayer(
    name="FIXTURE")


INTEGRATION = IntegrationTesting(
    bases=(FIXTURE,),
    name="INTEGRATION")


FUNCTIONAL = FunctionalTesting(
    bases=(FIXTURE,),
    name="FUNCTIONAL")


ACCEPTANCE = FunctionalTesting(bases=(FIXTURE,
                                      AUTOLOGIN_LIBRARY_FIXTURE,
                                      z2.ZSERVER_FIXTURE),
                               name="ACCEPTANCE")


class IntegrationTestCase(unittest.TestCase):
    """Base class for integration tests."""

    layer = INTEGRATION

    def setUp(self):
        super(IntegrationTestCase, self).setUp()
        self.portal = self.layer['portal']


class FunctionalTestCase(unittest.TestCase):
    """Base class for functional tests."""

    layer = FUNCTIONAL



# -*- coding: utf-8 -*-
"""Init and utils."""

from zope.i18nmessageid import MessageFactory
from plone import api

_ = MessageFactory('collective.behavior.talcondition')

PLONE_VERSION = int(api.env.plone_version()[0])


def initialize(context):
    """Initializer called when used as a Zope 2 product."""



# -*- coding: utf-8 -*-
from zope.component import adapts

from zope.interface import implements
from archetypes.schemaextender.interfaces import IBrowserLayerAwareExtender
from archetypes.schemaextender.interfaces import ISchemaExtender
from archetypes.schemaextender.field import ExtensionField


from Products.Archetypes.public import MultiSelectionWidget
from Products.Archetypes.public import LinesField
from Products.Archetypes.public import StringField
from Products.Archetypes.public import StringWidget

from collective.behavior.talcondition.interfaces import ICollectiveBehaviorTalconditionLayer
from collective.behavior.talcondition.interfaces import ITALConditionable


class TALConditionStringField(ExtensionField, StringField):
    """A string field that will contain an eventual TAL condition expression."""


class TALConditionLinesField(ExtensionField, LinesField):
    """A Lines field that will contain all roles
       that will bypass the tal condition."""


class TALConditionExtender(object):
    """TALCondition class"""

    implements(ISchemaExtender, IBrowserLayerAwareExtender)

    adapts(ITALConditionable)

    layer = ICollectiveBehaviorTalconditionLayer

    fields = [
        TALConditionStringField(
            'tal_condition',
            required=False,
            default='',
            searchable=False,
            languageIndependent=True,
            widget=StringWidget(
                label=(u"TAL condition expression"),
                description=(u'Enter a TAL expression that once evaluated '
                             'will return \'True\' if content should be '
                             'available. Elements \'member\', \'context\' '
                             'and \'portal\' are available for the '
                             'expression.'),
                i18n_domain='collective.behavior.talcondition',
                size="80",
            ),
        ),
        TALConditionLinesField(
            'roles_bypassing_talcondition',
            required=False,
            searchable=False,
            languageIndependent=True,
            widget=MultiSelectionWidget(
                size=10,
                label=(u'Roles that will bypass the TAL condition'),
                description=(u'Choose the different roles for which the TAL '
                             'condition will not be evaluated and always '
                             'considered \'True\'.'
                             ),
                i18n_domain='collective.behavior.talcondition',
            ),
            enforceVocabulary=True,
            multiValued=1,
            vocabulary_factory='plone.app.vocabularies.Roles',
        ),
    ]

    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self.fields



import os
import unittest
import robotsuite
from plone.testing import layered

from ..testing import ACCEPTANCE


def test_suite():
    suite = unittest.TestSuite()
    current_dir = os.path.abspath(os.path.dirname(__file__))
    robot_dir = os.path.join(current_dir, 'robot')
    robot_tests = [
        os.path.join('robot', doc) for doc in os.listdir(robot_dir)
        if doc.endswith('.robot') and doc.startswith('test_')
    ]
    for test in robot_tests:
        suite.addTests([
            layered(
                robotsuite.RobotTestSuite(test),
                layer=ACCEPTANCE
            ),
        ])
    return suite



# -*- coding: utf-8 -*-
"""Setup/installation tests for this package."""

from collective.behavior.talcondition.testing import IntegrationTestCase
from plone import api


class TestInstall(IntegrationTestCase):
    """Test installation of collective.behavior.talcondition into Plone."""

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if collective.behavior.talcondition is installed with portal_quickinstaller."""
        self.assertTrue(self.installer.isProductInstalled('collective.behavior.talcondition'))

    def test_uninstall(self):
        """Test if collective.behavior.talcondition is cleanly uninstalled."""
        self.installer.uninstallProducts(['collective.behavior.talcondition'])
        self.assertFalse(self.installer.isProductInstalled('collective.behavior.talcondition'))

    # browserlayer.xml
    def test_browserlayer(self):
        """Test that ICollectiveBehaviorTalconditionLayer is registered."""
        from collective.behavior.talcondition.interfaces import ICollectiveBehaviorTalconditionLayer
        from plone.browserlayer import utils
        self.assertIn(ICollectiveBehaviorTalconditionLayer, utils.registered_layers())



# -*- coding: utf-8 -*-
import unittest
from zope.interface import alsoProvides
from plone.app.testing import login
from plone.app.testing import TEST_USER_NAME
from collective.behavior.talcondition.testing import IntegrationTestCase
from collective.behavior.talcondition.behavior import ITALCondition
from collective.behavior.talcondition.interfaces import ITALConditionable
from collective.behavior.talcondition.utils import applyExtender
from collective.behavior.talcondition.utils import evaluateExpressionFor
from collective.behavior.talcondition.utils import _evaluateExpression
from collective.behavior.talcondition import PLONE_VERSION


class TestUtils(IntegrationTestCase):

    def setUp(self):
        """ """
        super(TestUtils, self).setUp()
        # create a testitem
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id='testitem',
                                  type_name='testtype',
                                  title='Test type')
        self.adapted = ITALCondition(self.portal.testitem)

    def test_wrong_condition(self):
        """In case the condition is wrong, it just returns False
           and a message is added to the Zope log."""
        # using a wrong expression does not break anything
        self.adapted.tal_condition = u'python: context.some_unexisting_method()'
        self.assertFalse(self.adapted.evaluate())

    @unittest.skipIf(PLONE_VERSION >= 5, 'Archetypes extender test skipped in Plone 5')
    def test_apply_extender(self):
        """Test that existing objects are correctly updated
           after enabling extender for their meta_type."""
        # the extender is not enabled for "Folder"
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id='testfolder',
                                  type_name='Folder',
                                  title='Test folder')
        testfolder = self.portal.testfolder
        self.assertFalse(hasattr(testfolder, 'tal_condition'))
        self.assertFalse(ITALConditionable.providedBy(testfolder))
        # enable the extender for testfolder
        alsoProvides(testfolder, ITALConditionable)
        # the schema is not updated until we do it
        self.assertFalse(hasattr(testfolder, 'tal_condition'))
        applyExtender(self.portal, meta_types=('ATFolder', ))
        # now the field is available
        self.assertTrue(hasattr(testfolder, 'tal_condition'))

    def test_empty_condition(self):
        # using an empty expression is considered True
        self.adapted.tal_condition = None
        self.assertTrue(self.adapted.evaluate())

    def test_bypass_for_manager(self):
        """In this case, no matter the expression is False,
           it will return True if current user is 'Manager'."""
        # using a wrong expression does not break anything
        self.adapted.tal_condition = "python:False"
        self.assertFalse(evaluateExpressionFor(self.adapted))
        self.adapted.roles_bypassing_talcondition = [u'Manager']
        # as current user is Manager, he can bypass the expression result
        self.assertTrue(evaluateExpressionFor(self.adapted))

    def test_extra_expr_ctx(self):
        """It is possible to pass extra values that will be available
           in the context of the expression."""
        self.adapted.tal_condition = "python: value == '122'"
        self.assertFalse(evaluateExpressionFor(self.adapted))
        self.assertTrue(evaluateExpressionFor(self.adapted, {'value': '122'}))

    def test_empty_expr_is_true(self):
        """Test parameter used by utils._evaluateExpression making an empty
           expression to be considered True or False."""
        # True by default
        self.assertTrue(_evaluateExpression(self.portal,
                                            expression=''))
        self.assertFalse(_evaluateExpression(self.portal,
                                             expression='',
                                             empty_expr_is_true=False))



# -*- coding: utf-8 -*-
import unittest
import logging
from Products.CMFCore.Expression import Expression, createExprContext
from plone import api
from collective.behavior.talcondition import PLONE_VERSION

logger = logging.getLogger('collective.behavior.talcondition')
WRONG_TAL_CONDITION = "The TAL expression '{0}' for element at '{1}' is wrong.  Original exception : {2}"


def evaluateExpressionFor(obj, extra_expr_ctx={}, error_pattern=WRONG_TAL_CONDITION):
    """Evaluate the expression stored in 'tal_condition' of given p_obj.
    """
    # get tal_condition
    tal_condition = obj.tal_condition and obj.tal_condition.strip() or ''

    roles_bypassing_talcondition = obj.roles_bypassing_talcondition

    if hasattr(obj, 'context'):
        obj = obj.context
    return _evaluateExpression(obj,
                               expression=tal_condition,
                               roles_bypassing_expression=roles_bypassing_talcondition,
                               extra_expr_ctx=extra_expr_ctx,
                               error_pattern=error_pattern)


def _evaluateExpression(obj,
                        expression,
                        roles_bypassing_expression=[],
                        extra_expr_ctx={},
                        empty_expr_is_true=True,
                        error_pattern=WRONG_TAL_CONDITION):
    """Evaluate given p_expression extending expression context with p_extra_expr_ctx."""
    if not expression.strip():
        return empty_expr_is_true

    res = True
    member = api.user.get_current()
    for role in roles_bypassing_expression or []:
        if member.has_role(str(role), obj):
            return res
    portal = api.portal.get()
    ctx = createExprContext(obj.aq_inner.aq_parent,
                            portal,
                            obj)
    ctx.setGlobal('member', member)
    ctx.setGlobal('context', obj)
    ctx.setGlobal('portal', portal)
    for extra_key, extra_value in extra_expr_ctx.items():
        ctx.setGlobal(extra_key, extra_value)
    try:
        res = Expression(expression)(ctx)
    except Exception, e:
        logger.warn(error_pattern.format(
            expression, obj.absolute_url(), str(e)))
        res = False
    return res


@unittest.skipIf(PLONE_VERSION >= 5, 'Archetypes extender skipped in Plone 5')
def applyExtender(portal, meta_types):
    """
      We add some fields using archetypes.schemaextender to every given p_meta_types.
    """
    logger.info("Adding talcondition fields : updating the schema for meta_types %s" % ','.join(meta_types))
    at_tool = api.portal.get_tool('archetype_tool')
    catalog = api.portal.get_tool('portal_catalog')
    catalog.ZopeFindAndApply(portal,
                             obj_metatypes=meta_types,
                             search_sub=True,
                             apply_func=at_tool._updateObject)
    logger.info("Done!")



# -*- coding: utf-8 -*-
import unittest
from plone.app.testing import login
from plone.app.testing import TEST_USER_NAME
from collective.behavior.talcondition.interfaces import ITALConditionable
from collective.behavior.talcondition.testing import IntegrationTestCase
from collective.behavior.talcondition.utils import evaluateExpressionFor
from collective.behavior.talcondition import PLONE_VERSION


class TestExtender(IntegrationTestCase):

    @unittest.skipIf(PLONE_VERSION >= 5, 'Archetypes extender test skipped in Plone 5')
    def test_extender(self):
        """The extender is enabled on ATDocument in testing.zcml.
           Check that 'tal_condition' is available."""
        login(self.portal, TEST_USER_NAME)
        self.portal.invokeFactory(id='doc',
                                  type_name='Document',
                                  title='Test document')
        doc = self.portal.doc
        self.assertTrue(ITALConditionable.providedBy(doc))
        # set a tal_condition and evaluate
        # this is True
        doc.tal_condition = u"python:context.portal_type=='Document'"
        self.assertTrue(evaluateExpressionFor(doc))
        # this is False
        doc.tal_condition = u"python:context.portal_type=='unexisting_portal_type'"
        self.assertFalse(evaluateExpressionFor(doc))
